<dl class="divide-y divide-gray-100">
  {% for field, val in object %}
    <div class="py-6 px-4 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
      <dt class="text-sm font-medium leading-6 text-gray-900">{{ field|capfirst }}:</dt>
      <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
        {{ val }}
      </dd>
    </div>
  {% endfor %}
</dl>
