{% extends "base.html" %}

{% block content %}
  <h1>Are you sure you want to delete following {{ object_verbose_name }}?</h1>
  <p class="mt-4">{{ object }}</p>
  <div class="m-8">
    <form method="post" action="{{ delete_view_url }}">
      {% csrf_token %}
      {{ form }}
      <button type="submit"
              class="inline-flex items-center py-2 px-3 text-sm font-semibold text-white bg-red-600 rounded-md shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600">
        Delete
      </button>
    </form>
  </div>
{% endblock content %}
